/*
Copyright (c) 2014 DenshiKousakuSenka
 This software is released under the MIT License.
 http://opensource.org/licenses/mit-license.php
 */

#include "DKS_LSM303DLHC.h"

DKS::LSM303DLHC::LSM303DLHC::LSM303DLHC(const DKS::I2C::II2C *I2C)
	: m_i2c(I2C), m_PowerMode(DKS::LSM303DLHC::Low), m_AccOutputDataRate(DKS::LSM303DLHC::ACC_ODR_50_HZ),
	m_AccFullScale(DKS::LSM303DLHC::FULLSCALE_2G),m_AccHighResolution(DKS::LSM303DLHC::HighResolutionENABLE),
	m_AccBlockDataUpdate(DKS::LSM303DLHC::Continuos), m_AccEndianDataSelection(DKS::LSM303DLHC::LSB),
	m_MagTemperatureSensor(DKS::LSM303DLHC::TEMPSENSOR_ENABLE), m_MagOutputDataRate(DKS::LSM303DLHC::MAG_ODR_15_HZ),
	m_MagFullScale(DKS::LSM303DLHC::MAG_FS_1_9_GA), m_MagWorkingMode(DKS::LSM303DLHC::MAG_CONTINUOS_CONVERSION)
{
	// Acceleration
	switch (m_AccFullScale)
	{
	case FULLSCALE_2G:
		AccCoeff = 2.0f * 2.0f / 65535.0f;
		break;
	case FULLSCALE_4G:
		AccCoeff = 4.0f * 2.0f / 65535.0f;
		break;
	case FULLSCALE_8G:
		AccCoeff = 8.0f * 2.0f / 65535.0f;
		break;
	case FULLSCALE_16G:
		AccCoeff = 16.0f * 2.0f / 65535.0f;
		break;
	}
	uint8_t ctrl1 = (uint8_t) (m_PowerMode | m_AccOutputDataRate | 0b111);
	uint8_t ctrl4 = (uint8_t) (m_AccBlockDataUpdate | m_AccEndianDataSelection | m_AccFullScale |m_AccHighResolution);

	m_i2c->Write(DKS::LSM303DLHC::ACC_I2C_ADDRESS, 0x20, 1, &ctrl1, 1);	//0x20 = CTRL_REG1_A
	m_i2c->Write(DKS::LSM303DLHC::ACC_I2C_ADDRESS, 0x23, 1, &ctrl4, 1);	//0x23 = CTRL_REG4_A

	// Mag
	uint8_t reg[3] = {0};
	reg[0] = (uint8_t)(m_MagTemperatureSensor| m_MagOutputDataRate);
	reg[1] = (uint8_t)m_MagFullScale;
	reg[2] = (uint8_t)m_MagWorkingMode;
	m_i2c->Write(DKS::LSM303DLHC::MAG_I2C_ADDRESS, 0x00, 1, reg, 3);	//0x00 = CRA_REG_M

	switch (m_MagFullScale)//MȂ
	{
	case MAG_FS_1_3_GA:
		MagCoeffXY 	= 1.0 / 1100.0;
		MagCoeffZ 	= 1.0 / 980.0;
		break;
	case MAG_FS_1_9_GA:
		MagCoeffXY 	= 1.0 / 855.0;
		MagCoeffZ 	= 1.0 / 760.0;
		break;
	case MAG_FS_2_5_GA:
		MagCoeffXY 	= 1.0 / 670.0;
		MagCoeffZ 	= 1000.0 / 600.0;
		break;
	case MAG_FS_4_0_GA:
		MagCoeffXY 	= 1.0 / 450.0;
		MagCoeffZ 	= 1.0 / 400.0;
		break;
	case MAG_FS_4_7_GA:
		MagCoeffXY 	= 1.0 / 400.0;
		MagCoeffZ 	= 1.0 / 355.0;
		break;
	case MAG_FS_5_6_GA:
		MagCoeffXY 	= 1.0 / 330.0;
		MagCoeffZ 	= 1.0 / 295.0;
		break;
	case MAG_FS_8_1_GA:
		MagCoeffXY 	= 2.0 / 230.0;
		MagCoeffZ 	= 2.0 / 205.0;
		break;
	}
}

void DKS::LSM303DLHC::LSM303DLHC::WaitTillDeviceReady(void)const
{
	m_i2c->WaitTillDeviceReady(DKS::LSM303DLHC::ACC_I2C_ADDRESS);
	m_i2c->WaitTillDeviceReady(DKS::LSM303DLHC::MAG_I2C_ADDRESS);
}

void DKS::LSM303DLHC::LSM303DLHC::ReadAccData(float* Data)const
{
	int16_t data[3];
	uint8_t buffer[6]={0};

	m_i2c->Read(DKS::LSM303DLHC::ACC_I2C_ADDRESS, 0x28 | (1<<7) , 1, buffer, 6);	//0x28=OUT_X_L_A AhX̐擪1bit1Auto Increment

	data[0]=(int16_t)(((uint16_t)buffer[1] << 8) + buffer[0]);
	data[1]=(int16_t)(((uint16_t)buffer[3] << 8) + buffer[2]);
	data[2]=(int16_t)(((uint16_t)buffer[5] << 8) + buffer[4]);

	Data[0] = AccCoeff * data[0];
	Data[1] = AccCoeff * data[1];
	Data[2] = AccCoeff * data[2];
}

void DKS::LSM303DLHC::LSM303DLHC::ReadAccData(uint8_t* Data)const
{
	m_i2c->Read(DKS::LSM303DLHC::ACC_I2C_ADDRESS, 0x28 | (1<<7) , 1, Data, 6);		//0x28=OUT_X_L_A AhX̐擪1bit1Auto Increment
}

void DKS::LSM303DLHC::LSM303DLHC::ReadTemperature(float &Temperature)const
{
	if (m_MagTemperatureSensor != DKS::LSM303DLHC::TEMPSENSOR_ENABLE)
	{
		Temperature = 0.0f;
	}
	else
	{
		uint8_t buffer[2]={0};
		m_i2c->Read(DKS::LSM303DLHC::MAG_I2C_ADDRESS, 0x31, 1, buffer, 2);	//0x31 =TEMP_OUT_H_M  Aœǂ߂Ȃ̂ł́HH
		int16_t temp = (int16_t)((uint16_t)buffer[0]<<4) | (buffer[1]>>4);

		Temperature = 0.25f * temp;
	}
}

void DKS::LSM303DLHC::LSM303DLHC::ReadMagData(uint8_t* Data)const
{
	m_i2c->Read(DKS::LSM303DLHC::MAG_I2C_ADDRESS, 0x03, 1, &Data[3], 6);	//0x03=OUT_X_H_M
}

void DKS::LSM303DLHC::LSM303DLHC::ReadMagData(float* Data)const
{
	int16_t data[3] = {0};
	uint8_t buffer[6] = {0};

	m_i2c->Read(DKS::LSM303DLHC::MAG_I2C_ADDRESS, 0x03, 1, buffer, 6);	//0x03=OUT_X_H_M

	data[0]=(int16_t)(((uint16_t)buffer[0] << 8) | buffer[1]);
	data[1]=(int16_t)(((uint16_t)buffer[4] << 8) | buffer[5]);
	data[2]=(int16_t)(((uint16_t)buffer[2] << 8) | buffer[3]);

	Data[0] = MagCoeffXY * data[0];
	Data[1] = MagCoeffXY * data[1];
	Data[2] = MagCoeffZ * data[2];

}
