/*
Copyright (c) 2014 DenshiKousakuSenka
 This software is released under the MIT License.
 http://opensource.org/licenses/mit-license.php
 */

#pragma once
#include "DKS_I2C_Base.h"

namespace DKS
{
	namespace LSM303DLHC
	{
		const uint8_t ACC_I2C_ADDRESS = 0x32;
		const uint8_t MAG_I2C_ADDRESS = 0x3C;

		enum PowerMode
		{
			Normal = 0x00,
			Low = 0x08
		};

		enum AccOutputDataRate
		{
			ACC_ODR_1_HZ         = 0x10,  // Output Data Rate = 1 Hz
			ACC_ODR_10_HZ        = 0x20,  // Output Data Rate = 10 Hz
			ACC_ODR_25_HZ        = 0x30,  // Output Data Rate = 25 Hz
			ACC_ODR_50_HZ        = 0x40,  // Output Data Rate = 50 Hz
			ACC_ODR_100_HZ       = 0x50,  // Output Data Rate = 100 Hz
			ACC_ODR_200_HZ       = 0x60,  // Output Data Rate = 200 Hz
			ACC_ODR_400_HZ       = 0x70,  // Output Data Rate = 400 Hz
			ACC_ODR_1620_HZ_LP   = 0x80,  // Output Data Rate = 1620 Hz only in Low Power Mode
			ACC_ODR_1344_HZ      = 0x90,  // Output Data Rate = 1344 Hz in Normal mode and 5376 Hz in Low Power Mode
		};

		enum AccFullScale
		{
			FULLSCALE_2G   = 0x00,
			FULLSCALE_4G   = 0x10,
			FULLSCALE_8G   = 0x20,
			FULLSCALE_16G  = 0x30
		};

		enum AccHighResolution
		{
			HighResolutionENABLE	= 0x08,
			HighResolutionDISABLE	= 0x00
		};

		enum AccBlockDataUpdate
		{
			Continuos	= 0x00,
			Single	= 0x80	//output registers not updated until MSB and LSB reading
		};

		enum AccEndianDataSelection
		{
			LSB	= 0x00,	//data LSB at lower address
			MSB	= 0x40	//data MSB at lower address
		};

		enum MagTemperatureSensor
		{
			TEMPSENSOR_ENABLE	= 0x80,
			TEMPSENSOR_DISABLE	= 0x00
		};

		enum MagOutputDataRate
		{
			MAG_ODR_0_75_HZ   = 0x00,  // Output Data Rate = 0.75 Hz
			MAG_ODR_1_5_HZ    = 0x04,  // Output Data Rate = 1.5 Hz
			MAG_ODR_3_0_HZ    = 0x08,  // Output Data Rate = 3 Hz
			MAG_ODR_7_5_HZ    = 0x0C,  // Output Data Rate = 7.5 Hz
			MAG_ODR_15_HZ     = 0x10,  // Output Data Rate = 15 Hz
			MAG_ODR_30_HZ     = 0x14,  // Output Data Rate = 30 Hz
			MAG_ODR_75_HZ     = 0x18,  // Output Data Rate = 75 Hz
			MAG_ODR_220_HZ    = 0x1C,  // Output Data Rate = 220 Hz
		};

		enum MagFullScale
		{
			MAG_FS_1_3_GA   = 0x20,  // 1.3 Gauss
			MAG_FS_1_9_GA   = 0x40,  // 1.9 Gauss
			MAG_FS_2_5_GA   = 0x60,  // 2.5 Gauss
			MAG_FS_4_0_GA   = 0x80,  // 4.0 Gauss
			MAG_FS_4_7_GA   = 0xA0,  // 4.7 Gauss
			MAG_FS_5_6_GA   = 0xC0,  // 5.6 Gauss
			MAG_FS_8_1_GA   = 0xE0,  // 8.1 Gauss
		};

		enum MagWorkingMode
		{
			MAG_CONTINUOS_CONVERSION = 0x00,
			MAG_SINGLE_CONVERSION    = 0x01,
			MAG_SLEEP                = 0x02,
		};

		class LSM303DLHC
		{
			float AccCoeff, MagCoeffXY, MagCoeffZ;
			const DKS::I2C::II2C *m_i2c;
			const PowerMode m_PowerMode;
			const AccOutputDataRate m_AccOutputDataRate;
			const AccFullScale m_AccFullScale;
			const AccHighResolution m_AccHighResolution;
			const AccBlockDataUpdate m_AccBlockDataUpdate;
			const AccEndianDataSelection m_AccEndianDataSelection;
			const MagTemperatureSensor m_MagTemperatureSensor;
			const MagOutputDataRate m_MagOutputDataRate;
			const MagFullScale m_MagFullScale;
			const MagWorkingMode m_MagWorkingMode;
		public:
			LSM303DLHC(const DKS::I2C::II2C *I2C);
			void WaitTillDeviceReady(void)const;
			void ReadAccData(float* Data)const;	//XYZ size=3
			void ReadAccData(uint8_t* Data)const;	//XYZ size=6 raw data
			void ReadTemperature(float &Temperature)const;
			void ReadMagData(float* Data)const;	//XYZ size=3
			void ReadMagData(uint8_t* Data)const;	//XYZ size=6 raw data
		};
	}
}

